#include "renderer.h"
#include <gtkmm/treeviewcolumn.h>


/****************************Spin Renderer********************/
Spin_CellRenderer::Spin_CellRenderer(double value, double lower,  double upper, double step_increment , double page_increment, double page_size)
{
    v=value;l=lower;u=upper;si=step_increment;pi=page_increment;ps=page_size;
}


void Spin_CellRenderer::on_edited(Gtk::Entry* entry, Glib::ustring path)
{
    Gtk::SpinButton* spin=(Gtk::SpinButton*)entry;
    double max,min;
    spin->get_range(min,max);
    double new_value=atof(spin->get_text().c_str());
    if(new_value>max) new_value=max;
    if(new_value<min) new_value=min;
    Glib::ustring text=property_text();
    double old_value=atof(text.c_str());
    if(new_value!=old_value)
    {
        char buf[32];
        sprintf(buf, "%f", new_value);
        edited(path, buf);
    }
}


Gtk::CellEditable* Spin_CellRenderer::start_editing_vfunc(
	GdkEvent* event, Gtk::Widget& widget,
	const Glib::ustring& path,
	const Gdk::Rectangle& background_area,
	const Gdk::Rectangle& cell_area,
	Gtk::CellRendererState flags)
{
    if(!property_editable()) return NULL;
    Gtk::Adjustment* padjustment=Gtk::manage(new Gtk::Adjustment(v,l,u,si,pi,ps));
    Gtk::SpinButton* spin=Gtk::manage(new Gtk::SpinButton(*padjustment));
    spin->set_has_frame(false);
    Glib::ustring text=property_text();
    spin->set_value(atoi(text.c_str()));
    spin->select_region(0,-1);
    spin->show();
    spin->set_numeric();
    spin->signal_editing_done().connect(bind(SigC::slot(*this,&Spin_CellRenderer::on_edited),spin,path));
    return spin;
}


/****************************Combo Renderer********************/
void Combo_CellRenderer::add_item(const Glib::ustring& item)
{
    items.push_back(item);
}


void Combo_CellRenderer::clear()
{
    items.clear();
}


void Combo_CellRenderer::on_edited(Gtk::Entry* entry, Glib::ustring path)
{
    Glib::ustring new_text=entry->get_text();
	//This piece of codes is broken, since sometimes the return value
	//of property_text() is not the original value of the cell as I expected.
	//Why???
    /*using namespace std;
		Glib::ustring old_text=property_text(); 
		cout<<old_text<<":"<<new_text<<endl;    
		if(new_text.compare(old_text)!=0)
		{
			edited(path, new_text);
		}
	*/
    edited(path, new_text);

    column->set_max_width(85); // shrink column back again.
}


Gtk::CellEditable* Combo_CellRenderer::start_editing_vfunc(
	GdkEvent* event, Gtk::Widget& widget,
	const Glib::ustring& path,
	const Gdk::Rectangle& background_area,
	const Gdk::Rectangle& cell_area,
	Gtk::CellRendererState flags)

{
    if(!property_editable()) return NULL;

    column->set_min_width(100); // expand column so we can see complete combo widget.

    Combo_Editable *combo=Gtk::manage(new Combo_Editable(items));
    Glib::ustring text=property_text();
    Gtk::Entry* entry=combo->get_entry();
    entry->set_text(text);
    entry->select_region(0,-1);
    entry->signal_editing_done().connect(bind(SigC::slot(*this,&Combo_CellRenderer::on_edited),entry,path));
    combo->show();
    return combo;
}
