#include <gtkmm.h>
//Two cell renderers are implemented.

//A helper function for adding the custom renderer. The use of a private function is a hacking, which is not good.
template <class ColumnType> inline
void treeView_add_column(Gtk::TreeView& treeview, const Glib::ustring& title, Gtk::CellRendererText* render, const Gtk::TreeModelColumn<ColumnType>& model_column)
{
    int cols_count=treeview.append_column(title, *render);
    Gtk::TreeViewColumn* pColumn = treeview.get_column(cols_count-1);
    if(pColumn) pColumn->add_attribute(render->property_text(),model_column);
    Gtk::TreeView_Private::_connect_auto_store_editable_signal_handler(&treeview,render,model_column);
};

class Spin_CellRenderer: public Gtk::CellRendererText
{
    public:
        Spin_CellRenderer(double value=1, double lower=1,  double upper=100000, double step_increment = 1, double page_increment = 10, double page_size = 0);
    private:
        double v,l,u,si,pi,ps;
    protected:
        virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags);
        virtual void on_edited(Gtk::Entry* entry,Glib::ustring path);
};

class Combo_CellRenderer: public Gtk::CellRendererText
{
    public:
        void add_item(const Glib::ustring& str);
        void clear();        
	void set_column(Gtk::TreeViewColumn *column){this->column = column;};
    private:
        std::list<Glib::ustring> items;
	Gtk::TreeViewColumn *column;
    protected:
        virtual Gtk::CellEditable* start_editing_vfunc(GdkEvent* event, Gtk::Widget& widget, const Glib::ustring& path, const Gdk::Rectangle& background_area, const Gdk::Rectangle& cell_area, Gtk::CellRendererState flags);
        virtual void on_edited(Gtk::Entry* entry,Glib::ustring path);	    
};

class Combo_Editable: public Gtk::EventBox,public Gtk::CellEditable

{
    private:
        Gtk::Combo combo_;
    public:
        Combo_Editable(std::list<Glib::ustring>& items):
        Glib::ObjectBase  (typeid(Combo_Editable)),
            Gtk::EventBox(),
            Gtk::CellEditable ()
        {
            get_entry()->set_has_frame(false);
            combo_.set_popdown_strings(items);
            combo_.show();
            add(combo_);
        }
        Gtk::Entry* get_entry(){return combo_.get_entry();}
        void set_editable(bool editable){get_entry()->set_editable(editable);}
    protected:
        virtual void start_editing_vfunc(GdkEvent* event){get_entry()->start_editing(event);}
        virtual void on_editing_done(){get_entry()->editing_done();}
};
